@@ -6,7 +6,9 @@ use namespace::clean -except => 'meta';
 
 BEGIN { extends qw(Catalyst::Model::KiokuDB) }
 
-__PACKAGE__->config->{dsn} = "hash";
+__PACKAGE__->config( dsn => "hash" );
+
+__PACKAGE__->config( model_class => "KiokuX::Model" );
 
 __PACKAGE__
 
